package	jp.co.sds_corp.util.window;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;

import jp.co.sds_corp.util.common.CmnLocale;

/**
* システムで有効なフォントの中から、フォントを選択するコントロールです。
*/
public class FontChooser extends JComponent
{
	JDialog		dialog;
	//JPanel panel = new JPanel();
	JOptionPane	pane;
	boolean		bOk = false;
	Font		font, cur;
	String[]	size = {
		"8", "10", "12", "14", "16", "18", "20", "24", "36", "48", "72"
	};
	String[]	logicalFonts = {
		"*Serif", "*SansSerif", "*Monospaced", "*Dialog", "*DialogInput"
	};
	Vector<Font>	vecLogic = new Vector<Font>();
	JComboBox	CMB_FONT = new JComboBox( logicalFonts );
	JComboBox	CMB_SIZE = new JComboBox( size );
	JRadioButton	RDO_PLAIN;
	JRadioButton	RDO_ITALIC;
	JCheckBox	CHK_BOLD;
	JTextField	TXT_SAMPLE;
	JPanel		btnPnl;
	JButton		BTN_OK;
	JButton		BTN_CANCEL;
	JButton		BTN_RESET;
	Vector<String>	vecFamily = new Vector<String>();
	Vector<Font>	vecFont = new Vector<Font>();
	ResourceBundle res;

	/**
	* FontChooserを生成します。
	*/
	public FontChooser() {
		createControl();
		setupFonts();
	}

	private void createControl() {
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ));

		res = ResourceBundle.getBundle( "resource.util_gui", CmnLocale.getLocale() );
		String text;
		text = res.getString( "font.plain.text" );
		RDO_PLAIN = new JRadioButton( text );
		text = res.getString( "font.italic.text" );
		RDO_ITALIC = new JRadioButton( text );
		text = res.getString( "font.bold.text" );
		CHK_BOLD = new JCheckBox( text );
		text = res.getString( "font.sample.text" );
		TXT_SAMPLE = new JTextField( text );

		Box	box4 = new Box( BoxLayout.X_AXIS );
		text = res.getString( "font.face.text" );
		box4.add( new JLabel( text ));
		box4.add( Box.createHorizontalStrut( 100 ));
		text = res.getString( "font.size.text" );
		box4.add( new JLabel( text ));
		this.add( box4 );

		Box	box1 = new Box( BoxLayout.X_AXIS );
		//CMB_FONT.setEditable( true );
		CMB_FONT.setMaximumRowCount( 5 );
		CMB_FONT.addItemListener( new ItemListener() {
			public void itemStateChanged( ItemEvent ev ) {
				fontChanged();
			}
		} );
		box1.add( CMB_FONT );
		Dimension dim = CMB_SIZE.getSize();
		dim.setSize( 50, dim.getHeight() );
		CMB_SIZE.setSize( dim );
		CMB_SIZE.setMaximumRowCount( 5 );
		CMB_SIZE.addItemListener( new ItemListener() {
			public void itemStateChanged( ItemEvent ev ) {
				fontChanged();
			}
		} );
		box1.add( CMB_SIZE );
		this.add( box1 );

		Box	box2 = new Box( BoxLayout.X_AXIS );
		ButtonGroup grp = new ButtonGroup();
		RDO_PLAIN.setSelected( true );
		RDO_PLAIN.addActionListener( new ActionListener() { 
			public void actionPerformed( ActionEvent ev ) {
				fontChanged();
			}
		} );
		grp.add( RDO_PLAIN );
		box2.add( RDO_PLAIN );
		RDO_ITALIC.setSelected( false );
		RDO_ITALIC.addActionListener( new ActionListener() { 
			public void actionPerformed( ActionEvent ev ) {
				fontChanged();
			}
		} );
		grp.add( RDO_ITALIC );
		box2.add( RDO_ITALIC );
		CHK_BOLD.setSelected( false );
		CHK_BOLD.addActionListener( new ActionListener() { 
			public void actionPerformed( ActionEvent ev ) {
				fontChanged();
			}
		} );
		box2.add( CHK_BOLD );
		this.add( box2 );

		TXT_SAMPLE.setEditable( false );
		TXT_SAMPLE.setHorizontalAlignment( JTextField.CENTER );
		this.add( TXT_SAMPLE );

		btnPnl = new JPanel();
		btnPnl.setLayout( new BoxLayout( btnPnl,  BoxLayout.X_AXIS ));
		BTN_OK = new JButton( "OK" );
		BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onOk();
			}
		} );
		btnPnl.add( BTN_OK );
		BTN_CANCEL = new JButton( "Cancel" );
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onCancel();
			}
		} );
		btnPnl.add( BTN_CANCEL );
		BTN_RESET = new JButton( "Reset" );
		BTN_RESET.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				onReset();
			}
		} );
		btnPnl.add( BTN_RESET );
	}

	private void setupFonts() {
		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		Font[] af = ge.getAllFonts();
		for ( int i = 0; i < af.length; i++ ) {
			vecFont.add( af[i] );
			String family = af[i].getFamily();
			if ( vecFamily.indexOf( family ) >= 0 ) {
				continue;
			}
			vecFamily.add( family );
			CMB_FONT.addItem( family );
		}
		for ( int i = 0; i < logicalFonts.length; i++ ) {
			Font f = new Font( logicalFonts[i].substring( 1 ), Font.PLAIN, 1 );
			vecLogic.add( f );
		}
	}

	private void fontChanged() {
		int size = Integer.valueOf( (String )CMB_SIZE.getSelectedItem() )
						.intValue();
		boolean bItalic = RDO_ITALIC.isSelected();
		boolean bBold = CHK_BOLD.isSelected();
		String sel = (String )CMB_FONT.getSelectedItem();

		int style = Font.PLAIN;
		if ( bItalic ) {
			style = Font.ITALIC;
		}
		if ( bBold ) {
			if ( style == Font.PLAIN ) {
				style = Font.BOLD;
			} else {
				style += Font.BOLD;
			}
		}

		Font f = null;
		boolean bFound = false;
		for ( int i = 0; i < vecFont.size(); i++ ) {
			f = (Font )vecFont.get( i );
			if ( f.getFamily().equals( sel )) {
				bFound = true;
				break;
			}
		}
		if ( !bFound ) {
			for ( int i = 0; i < logicalFonts.length; i++) {
				String l = logicalFonts[i];
				if ( sel.equals( l )) {
					f = (Font )vecLogic.get( i );
					bFound = true;
					break;
				}
			}
		}
		if ( bFound ) {
			cur = f.deriveFont( (float )size ).deriveFont( style );
			TXT_SAMPLE.setFont( cur );
			TXT_SAMPLE.setPreferredSize( new Dimension( 300, 36 ));
		} else {
			String msg = res.getString( "font.err1.message" );
			String title = res.getString( "font.err1.title" );
			JOptionPane.showMessageDialog( this,
				msg + " [" + sel + "]", title,
				JOptionPane.ERROR_MESSAGE );
		}
	}

	/**
	* モーダルなフォント選択ダイアログを表示します。
	* @param	parent	ダイアログの親Component。nullも可
	* @return	ユーザが選択したフォント。OKが押されなければnull
	*/
	public Font showDialog( Component parent ) {
		if ( parent instanceof JDialog ) {
			dialog = new JDialog( (JDialog )parent, true );
		} else
		if ( parent instanceof JFrame ) {
			dialog = new JDialog( (JFrame )parent, true );
		} else {
			dialog = new JDialog();
		}
		String text;
		text = res.getString( "font.title" );
		dialog.setTitle( text );
		dialog.setModal( true );
		dialog.setSize( new Dimension( 300, 150 ));
		Container content = dialog.getContentPane();
		content.setLayout( new BoxLayout( content, BoxLayout.Y_AXIS ));
		content.add( this );
		content.add( btnPnl );
		dialog.getRootPane().setDefaultButton( BTN_OK );
		setActionMap( dialog );

		AbstractAction escAct = new AbstractAction( "閉じる" ) {
			public void actionPerformed(ActionEvent evt) {
				dialog.dispose();
			}
		};
		KeyStroke escape =
			KeyStroke.getKeyStroke( KeyEvent.VK_ESCAPE, 0, false );
		InputMap imap = dialog.getRootPane()
						.getInputMap( JComponent.WHEN_IN_FOCUSED_WINDOW );
		imap.put( escape, "ESCAPE");
		dialog.getRootPane().getActionMap().put( "ESCAPE", escAct );

		if ( parent != null ) {
			Rectangle rect = SwingUtilities.getRoot( parent ).getBounds();
			Dimension d = dialog.getSize();
			dialog.setLocation(
				(int )( rect.getX() + rect.getWidth() / 2 - d.getWidth() / 2 ),
				(int )( rect.getY() + rect.getHeight() / 2 - d.getHeight() / 2 ));
		}

		bOk = false;
		dialog.setVisible( true );
		if ( isOk()) {
			return cur;
		}
		return null;
	}

	private void setActionMap( JDialog dialog ) {
		AbstractAction act = new AbstractAction( "了解" ) {
			public void actionPerformed( ActionEvent evt ) {
				onCancel();
			}
		};
		InputMap imap = dialog.getRootPane().getInputMap();
		imap.put( KeyStroke.getKeyStroke( KeyEvent.VK_ESCAPE, 0 ), "close-it" );
		dialog.getRootPane().getActionMap().put( "close-it", act );
	}

	private void onOk() {
		bOk = true;
		if ( dialog != null ) {
			dialog.setVisible( false );
		}
	}

	private void onCancel() {
		bOk = false;
		if ( dialog != null ) {
			dialog.setVisible( false );
		}
	}

	private void onReset() {
		cur = font;
		setupControl( font );
	}

	/**
	* ダイアログでOKが選択されたかどうかを返します。
	* @return	OKが押されていればtrue、それ以外はfalse
	*/
	public boolean isOk() {
		return bOk;
	}

	/**
	* FontChooserのフォントを設定します。
	* @param	f		設定するフォント
	*/
	public void setSelectedFont( Font f ) {
		font = f;
		cur = f;
		setupControl( f );
	}
	private void setupControl( Font f ) {
		String sel = f.getFamily();
		for ( int i = 0; i < logicalFonts.length; i++ ) {
			if ( logicalFonts[i].substring( 1 ).equals( sel )) {
				sel = "*" + sel;
				break;
			}
		}
		CMB_FONT.setSelectedItem( sel );
		CMB_SIZE.setSelectedItem( String.valueOf( f.getSize()) );
		if ( f.isBold()) {
			CHK_BOLD.setSelected( true );
		} else {
			CHK_BOLD.setSelected( false );
		}
		if ( f.isItalic()) {
			RDO_ITALIC.setSelected( true );
		} else {
			RDO_PLAIN.setSelected( true );
		}
		// TXT_SAMPLE.setFont( f );
		fontChanged();
	}
	/**
	* FontChooserで選択されているFontを返します。
	* @return	選択されているFont
	*/
	public Font getSelectedFont() {
		return cur;
	}

}
