package jp.co.sds_corp.util.window;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.HashMap;
import java.util.Map;
import java.util.ResourceBundle;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.util.common.CmnLocale;

/**
* Mail送信に関する設定をするPanelです。
* @author	小見山　聡
*/
public class MailSetupPanel extends JPanel implements ISetupPanel
{
	JCheckBox	CHK;
	JLabel		LBL1;
	JLabel		LBL2;
	JLabel		LBL3;
	JTextField	TXT1 = new JTextField();
	JTextField	TXT2 = new JTextField();
	JTextField	TXT3 = new JTextField();
	ResourceBundle	res;

	/**
	* このパネルを初期化します。
	*/
	public MailSetupPanel() {
		res = ResourceBundle.getBundle( "resource.util_gui", CmnLocale.getLocale() );
		String text;
		text = res.getString( "mail.title" );
		setBorder( new TitledBorder( text ));
		setLayout( new BorderLayout() );
		setPreferredSize( new Dimension( 100, 100 ));

		text = res.getString( "mail.check.text" );
		CHK = new JCheckBox( text );
		CHK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ev ) {
				JCheckBox chk = (JCheckBox )ev.getSource();
				setSelected( chk.getModel().isSelected());
			}
		});
		add( CHK, BorderLayout.NORTH );
		JPanel pnl1 = new JPanel();
		pnl1.setLayout( new GridLayout( 3, 2 ));

		text = res.getString( "mail.smtpserver.text" );
		LBL1 = new JLabel( text );
		text = res.getString( "mail.sendfrom.text" );
		LBL2 = new JLabel( text );
		text = res.getString( "mail.sendto.text" );
		LBL3 = new JLabel( text );
		TXT1.setPreferredSize( new Dimension( 100, 20 ));
		TXT2.setPreferredSize( new Dimension( 100, 20 ));
		TXT3.setPreferredSize( new Dimension( 100, 20 ));

		pnl1.add( LBL1 );
		pnl1.add( TXT1 );
		pnl1.add( LBL2 );
		pnl1.add( TXT2 );
		pnl1.add( LBL3 );
		pnl1.add( TXT3 );
		add( pnl1, BorderLayout.CENTER );

		TextFocusHandler tf = new TextFocusHandler();
		TXT1.addFocusListener( tf );
		TXT2.addFocusListener( tf );
		TXT3.addFocusListener( tf );
	}

	class TextFocusHandler extends FocusAdapter
	{
		public void focusGained( FocusEvent ev ) {
			JTextField txt = (JTextField ) ev.getSource();
			txt.selectAll();
		}
	}

	Map<String,String>	map;
	public void setData( Map<String,String> map ) {
		this.map = map;
		if ( map == null ) {
			setSelected( false );
			TXT1.setText( "" );
			TXT2.setText( "" );
			TXT3.setText( "" );
			return;
		}
		setSelected( true );
		TXT1.setText( (String )map.get( "Server" ));
		TXT2.setText( (String )map.get( "From" ));
		TXT3.setText( (String )map.get( "To" ));
	}

	public Map<String,String> getData() {
		if ( CHK.isSelected() ) {
			if ( map == null ) {
				map = new HashMap<String,String>();
			}
			map.put( "Type", "mail" );
			map.put( "Server", TXT1.getText() );
			map.put( "From", TXT2.getText() );
			map.put( "To", TXT3.getText() );
			return map;
		}
		return null;
	}

	public void setEnabled( boolean bEn ) {
		super.setEnabled( bEn );
		CHK.setEnabled( bEn );
		LBL1.setEnabled( bEn );
		LBL2.setEnabled( bEn );
		LBL3.setEnabled( bEn );
		TXT1.setEnabled( bEn );
		TXT2.setEnabled( bEn );
		TXT3.setEnabled( bEn );
	}

	public void setSelected( boolean bChk ) {
		CHK.setSelected( bChk );
		LBL1.setEnabled( bChk );
		LBL2.setEnabled( bChk );
		LBL3.setEnabled( bChk );
		TXT1.setEnabled( bChk );
		TXT2.setEnabled( bChk );
		TXT3.setEnabled( bChk );
	}
	public boolean isSelected() {
		return CHK.isSelected();
	}

	public void clear() {
		setSelected( false );
		TXT1.setText( null );
		TXT2.setText( null );
		TXT3.setText( null );
	}

	public String getHost() {
		return TXT1.getText();
	}
	public void setHost( String host ) {
		TXT1.setText( host );
	}
	public String getTo() {
		return TXT3.getText();
	}
	public void setTo( String to ) {
		TXT3.setText( to );
	}
	public String getFrom() {
		return TXT2.getText();
	}
	public void setFrom( String from ) {
		TXT2.setText( from );
	}
}

